Package: nnTensor
Type: Package
Title: Non-Negative Tensor Decomposition
Version: 1.4.0
Authors@R: c(person("Koki", "Tsuyuzaki", role = c("aut", "cre"),
                      email = "k.t.the-answer@hotmail.co.jp"),
               person("Itoshi", "Nikaido", role = "aut"))
Depends: R (>= 3.4.0)
Imports: methods, MASS, fields, rTensor, plot3D, tagcloud, ggplot2,
        Vicus
Suggests: knitr, rmarkdown, testthat, dplyr, tweedie
Description: Some functions for performing non-negative matrix factorization, non-negative CANDECOMP/PARAFAC (CP) decomposition, non-negative Tucker decomposition, and generating toy model data. See Andrzej Cichock et al (2009) and the reference section of GitHub README.md <https://github.com/rikenbit/nnTensor>, for details of the methods.
License: MIT + file LICENSE
URL: https://github.com/rikenbit/nnTensor
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-08 11:46:37 UTC; koki
Author: Koki Tsuyuzaki [aut, cre],
  Itoshi Nikaido [aut]
Maintainer: Koki Tsuyuzaki <k.t.the-answer@hotmail.co.jp>
Repository: CRAN
Date/Publication: 2026-05-08 12:30:09 UTC
Built: R 4.5.2; ; 2026-05-08 15:47:15 UTC; unix
