import(fields)
import(rTensor)
import(plot3D)
import(tagcloud)
import(ggplot2)

importFrom(grDevices, dev.off, png)
importFrom(methods, is)
importFrom(MASS, ginv)
importFrom(Vicus, graphMatrix)
importFrom(stats, runif, sd, rpois, median, dnorm, dpois)
importFrom(graphics, layout, plot.new)
importFrom("stats", "as.dist", "cophenetic", "cor", "hclust")
importFrom("utils", "setTxtProgressBar", "txtProgressBar")

S3method(plot, NMF)

export(
    toyModel,
    NMF,
    NMTF,
    siNMF,
    jNMF,
    plotTensor3D,
    plotTensor2D,
    recTensor,
    NTD,
    NTF,
    GabrielNMF,
    kFoldMaskTensor,
    ZITNMF,
    PSDTF
	)
