X <- toyModel("NMF")

out1 <- GabrielNMF(X, J=3, algorithm="Frobenius", num.iter=2)
out2 <- GabrielNMF(X, J=3, algorithm="KL", num.iter=2)
out3 <- GabrielNMF(X, J=3, algorithm="IS", num.iter=2)
out4 <- GabrielNMF(X, J=3, algorithm="Pearson", num.iter=2)
out5 <- GabrielNMF(X, J=3, algorithm="Hellinger", num.iter=2)
out6 <- GabrielNMF(X, J=3, algorithm="Neyman", num.iter=2)
out7 <- GabrielNMF(X, J=3, algorithm="Alpha", num.iter=2)
out8 <- GabrielNMF(X, J=3, algorithm="Beta", num.iter=2)

expect_equivalent(length(out1), 1)
expect_equivalent(length(out2), 1)
expect_equivalent(length(out3), 1)
expect_equivalent(length(out4), 1)
expect_equivalent(length(out5), 1)
expect_equivalent(length(out6), 1)
expect_equivalent(length(out7), 1)
expect_equivalent(length(out8), 1)
