X <- toyModel("NMF")

out1 <- NMF(X, num.iter=2, runtime=30, J=1:4, rank.method="all")
out2 <- NMF(X, num.iter=2, runtime=30, J=1:4, rank.method="ccc")
out3 <- NMF(X, num.iter=2, runtime=30, J=1:4, rank.method="dispersion")
out4 <- NMF(X, num.iter=2, runtime=30, J=1:4, rank.method="rss")
out5 <- NMF(X, num.iter=2, runtime=30, J=1:4, rank.method="evar")
out6 <- NMF(X, num.iter=2, runtime=30, J=1:4, rank.method="residuals")
out7 <- NMF(X, num.iter=2, runtime=30, J=1:4, rank.method="sparseness.basis")
out8 <- NMF(X, num.iter=2, runtime=30, J=1:4, rank.method="sparseness.coef")
out9 <- NMF(X, num.iter=2, runtime=30, J=1:4, rank.method="sparseness2.basis")
out10 <- NMF(X, num.iter=2, runtime=30, J=1:4, rank.method="sparseness2.coef")
out11 <- NMF(X, num.iter=2, runtime=30, J=1:4, rank.method="norm.info.gain.basis")
out12 <- NMF(X, num.iter=2, runtime=30, J=1:4, rank.method="norm.info.gain.coef")
out13 <- NMF(X, num.iter=2, runtime=30, J=1:4, rank.method="singular")
out14 <- NMF(X, num.iter=2, runtime=30, J=1:4, rank.method="volume")
out15 <- NMF(X, num.iter=2, runtime=30, J=1:4, rank.method="condition")

expect_equivalent(length(out1), 10)
expect_equivalent(length(out2), 10)
expect_equivalent(length(out3), 10)
expect_equivalent(length(out4), 10)
expect_equivalent(length(out5), 10)
expect_equivalent(length(out6), 10)
expect_equivalent(length(out7), 10)
expect_equivalent(length(out8), 10)
expect_equivalent(length(out9), 10)
expect_equivalent(length(out10), 10)
expect_equivalent(length(out11), 10)
expect_equivalent(length(out12), 10)
expect_equivalent(length(out13), 10)
expect_equivalent(length(out14), 10)
expect_equivalent(length(out15), 10)
